/*===========================================================================*
 * optic_feature.c                                                           *
 *                                                                           *
 *      Procedures to create or update OpticFeature Frame                    *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void createOpticFeatureFrame(GtkWidget *)                            *
 *      void updateOpticFeatureFrame()                                       * 
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include <gtk/gtk.h>
#include <math.h>
#include "cono_map.h"
#include "widget_set.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "intl.h"

/*=====================*
 * EXPORTED PROCEDURES *
 *=====================*/
void createOpticFeatureFrame(GtkWidget* parent);
void updateOpticFeatureFrame();

/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
extern GtkStyle *GreekStyle;
extern GtkTooltips *tooltips;

/*==================*
 * GLOBAL FUNCTIONS *   
 *==================*/
extern void changeOpticalAngle();
extern void updateStereoFrame();

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget *OpticFeatureFrame, *OpticFeatureBox = NULL;
static GtkWidget* OmegaSpin, *EpsSpin, *OpticSignLabel;
static GtkWidget *AlphaSpin, *GammaSpin, *V2ZSpin, *V2XSpin;

/*====================*
 * Internal Procedure *
 *====================*/
static void OpticFeatureUniaxial(), OpticFeatureBiaxial();
static void setOmega(GtkSpinButton*, gpointer);
static void setEps(GtkSpinButton*, gpointer);
static void setAlpha(GtkSpinButton*, gpointer);
static void setGamma(GtkSpinButton*, gpointer);
static void set2Vz(GtkSpinButton*, gpointer);
static void set2Vx(GtkSpinButton*, gpointer);


/*======================================================================*
 * EXPORTED PROCEDURES                                                  *
 *                                                                      *
 *               createOpticFeatureFrame                                *
 *               updateOpticFeatureFrame                                *
*=======================================================================*/
void createOpticFeatureFrame(GtkWidget* parent)
{
   OpticFeatureFrame = gtk_frame_new(_("(Optic Feature)"));
   gtk_container_set_border_width( GTK_CONTAINER(OpticFeatureFrame), 10);
   gtk_box_pack_start(GTK_BOX(parent), OpticFeatureFrame, FALSE, FALSE, 0);

   OpticFeatureBox  = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(OpticFeatureFrame), OpticFeatureBox);
   
   updateOpticFeatureFrame();
}

void updateOpticFeatureFrame()
{
   if(OpticFeatureBox != NULL)
     gtk_widget_destroy(OpticFeatureBox);
 
   OpticFeatureBox  = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(OpticFeatureFrame), OpticFeatureBox);
      
   if(drawing_method.axial_type == Uniaxial)
     OpticFeatureUniaxial();
   else if(drawing_method.axial_type == Biaxial)
     OpticFeatureBiaxial();
   gtk_widget_show_all(OpticFeatureBox);
}



/*======================================================================*
 * Internal Procedure                                                   *
 *                                                                      *
 *                     OpticFeatureUniaxial                             *
 *                     OpticFeatureBiaxial                              *
 *======================================================================*/
 
void
OpticFeatureUniaxial()
{
   GtkWidget *RILabel, *SignLabel;
   GtkWidget* RIBox = gtk_hbox_new(FALSE, 0);
   GtkWidget* SignBox = gtk_hbox_new(FALSE, 0);
   
   gtk_box_pack_start (GTK_BOX(OpticFeatureBox), RIBox, FALSE, FALSE, 9);
   gtk_box_pack_start (GTK_BOX(OpticFeatureBox), SignBox, FALSE, FALSE, 12);

   RILabel = pack_label(RIBox, _("RI"), 0);
   gtk_widget_set_usize (RILabel, 80, 0);

   OmegaSpin = pack_spin(RIBox, "w  ", 25, 60, 
			 parameter.omega, 1, 2.5, 0.001, 0.1, 3);
   EpsSpin = pack_spin(RIBox, "e  ", 25, 60,
			 parameter.epsilon, 1, 2.5, 0.001, 0.1, 3);
   parameter.RI = sqrt(parameter.epsilon*parameter.omega);

   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(OmegaSpin), "label"),
			GreekStyle);   
   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(EpsSpin), "label"),
			GreekStyle);   
   gtk_tooltips_set_tip (tooltips, OmegaSpin, _("Ordinary Ray Refractive Index"), NULL);
   gtk_tooltips_set_tip (tooltips, EpsSpin, _("Extraordinary Ray Refractive Index"), NULL);
   gtk_container_add(GTK_CONTAINER(RIBox), gtk_label_new("  "));

   SignLabel = pack_label(SignBox, _("Optic Sign :"), 0);
   gtk_widget_set_usize (SignLabel, 80, 0);

   if(parameter.omega < parameter.epsilon)
     OpticSignLabel = pack_label( SignBox, _("Positive"), 10);
   else if(parameter.omega > parameter.epsilon)
     OpticSignLabel = pack_label( SignBox,  _("Negative"), 10);
   else
     OpticSignLabel = pack_label( SignBox,  _("Isotropic"), 10); 
   gtk_misc_set_alignment(GTK_MISC(OpticSignLabel), 0, 0.5);

   gtk_signal_connect(GTK_OBJECT(OmegaSpin), "changed", 
		      GTK_SIGNAL_FUNC(setOmega), NULL);  
   gtk_signal_connect(GTK_OBJECT(EpsSpin), "changed", 
		      GTK_SIGNAL_FUNC(setEps), NULL);  

}

void
OpticFeatureBiaxial()
{
   GtkWidget *RILabel, *V2Label;
   GtkWidget *RIBox = gtk_hbox_new(FALSE, 0);
   GtkWidget *V2Box = gtk_hbox_new(FALSE, 0);

   gtk_box_pack_start (GTK_BOX(OpticFeatureBox), RIBox, FALSE, FALSE, 9);
   gtk_box_pack_start (GTK_BOX(OpticFeatureBox), V2Box, FALSE, FALSE, 9);

   RILabel = pack_label(RIBox, _("RI"), 0);
   gtk_widget_set_usize (RILabel, 80, 0);
  
   V2Label = pack_label(V2Box, _("Optic Angle"), 0);
   gtk_widget_set_usize (V2Label, 80, 0);
		   
   
   AlphaSpin = pack_spin(RIBox, "a  ", 25, 60, parameter.alpha, 1, 2.5, 
			 0.001, 0.1, 3);
   GammaSpin = pack_spin(RIBox, "g  ", 25, 60, parameter.gamma, 1, 2.5, 
			 0.001, 0.1, 3);
   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(AlphaSpin), "label"),
			GreekStyle);   
   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(GammaSpin), "label"),
			GreekStyle);   
   gtk_tooltips_set_tip (tooltips, AlphaSpin, 
			 _("Minimum Principal Refractive Index"), NULL);
   gtk_tooltips_set_tip (tooltips, GammaSpin, 
			 _("Maximum Principal Refractive Index"), NULL);
   parameter.RI = sqrt(parameter.alpha*parameter.gamma);
   gtk_container_add(GTK_CONTAINER(RIBox), gtk_label_new("  "));
   

   V2ZSpin = pack_spin(V2Box, "2Vz ", 35, 50, 
		       parameter.optic_axial_angle, 0, 180, 1, 10, 0);
   V2XSpin = pack_spin(V2Box, "2Vx ", 35, 50,
		       180-parameter.optic_axial_angle, 0, 180, 1, 10, 0);
   gtk_tooltips_set_tip (tooltips, V2ZSpin, 
			 _("Optic Angle around Z-Axis"), NULL);
   gtk_tooltips_set_tip (tooltips, V2XSpin, 
			 _("Optic Angle around X-Axis"), NULL);

   gtk_signal_connect(GTK_OBJECT(AlphaSpin), "changed", 
		      GTK_SIGNAL_FUNC(setAlpha), NULL);  
   gtk_signal_connect(GTK_OBJECT(GammaSpin), "changed", 
		      GTK_SIGNAL_FUNC(setGamma), NULL);  
   gtk_signal_connect(GTK_OBJECT(V2ZSpin), "changed", 
		      GTK_SIGNAL_FUNC(set2Vz), NULL);  
   gtk_signal_connect(GTK_OBJECT(V2XSpin), "changed", 
		      GTK_SIGNAL_FUNC(set2Vx), NULL);  
}


/*======================================================================*
* Callback Functions                                                    *
*                                                                       *
*                      setOmega                                         *
*                      setEps                                           *
*                      setAlpha                                         *
*                      setGamma                                         *
*                      set2Vz                                           *
*                      set2Vx                                           *
*=======================================================================*/

void setOmega(GtkSpinButton *spin, gpointer data)
{
   GtkWidget *parent = OpticSignLabel->parent;
   parameter.omega= gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   parameter.RI = sqrt(parameter.epsilon*parameter.omega);
   
   gtk_widget_destroy(OpticSignLabel);

   if(parameter.omega < parameter.epsilon)
     OpticSignLabel = pack_label( parent, _("Positive"), 10);
   else if(parameter.omega > parameter.epsilon)
     OpticSignLabel = pack_label( parent, _("Negative"), 10);
   else
     OpticSignLabel = pack_label( parent, _("Isotropic"), 10);
   gtk_misc_set_alignment(GTK_MISC(OpticSignLabel), 0, 0.5);
   gtk_widget_show_all(GTK_WIDGET(OpticFeatureBox));

   updateStereoFrame();
}

void setEps(GtkSpinButton *spin, gpointer data)
{
   GtkWidget *parent = OpticSignLabel->parent;
   parameter.epsilon=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   parameter.RI = sqrt(parameter.epsilon*parameter.omega);

   gtk_widget_destroy(OpticSignLabel);
   if(parameter.omega < parameter.epsilon)
     OpticSignLabel = pack_label( parent, _("Positive"), 10);
   else if(parameter.omega > parameter.epsilon)
     OpticSignLabel = pack_label( parent, _("Negative"), 10);
   else
     OpticSignLabel = pack_label( parent, _("Isotropic"), 10); 
   gtk_misc_set_alignment(GTK_MISC(OpticSignLabel), 0, 0.5);
   gtk_widget_show_all(GTK_WIDGET(OpticFeatureBox));

   updateStereoFrame();
}

void
setAlpha( GtkSpinButton* spin, gpointer data)
{
   GtkAdjustment* adj 
     = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(GammaSpin));

   parameter.alpha = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   parameter.RI = sqrt(parameter.alpha*parameter.gamma);

   adj->lower = parameter.alpha;
   gtk_spin_button_set_adjustment( GTK_SPIN_BUTTON(GammaSpin), adj);

   updateStereoFrame();
}

void
setGamma( GtkSpinButton* spin, gpointer data)
{
   GtkAdjustment* adj 
     = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(AlphaSpin));

   parameter.gamma = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   parameter.RI = sqrt(parameter.alpha*parameter.gamma);

   adj->upper = parameter.gamma;
   gtk_spin_button_set_adjustment( GTK_SPIN_BUTTON(AlphaSpin), adj);

   updateStereoFrame();
}

void
set2Vz( GtkSpinButton* spin, gpointer data)
{
   gfloat val = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(V2XSpin), 180-val);
   parameter.optic_axial_angle = val;
   
   changeOpticalAngle();
}

void
set2Vx( GtkSpinButton* spin, gpointer data)
{
   gfloat val = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(V2ZSpin), 180-val);
   parameter.optic_axial_angle = 180 - val;
   
   changeOpticalAngle();
}
