/*===========================================================================*
 * select_buttons.c                                                          *
 *                                                                           *
 *      Main menue for Conoscopic Image Drawing Method                       *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void createSelectButtonFrame(GtkWidget*)                             *
 *      void gtk_button_replace_label(GtkWidget *button, gchar* str)         *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <gtk/gtk.h>
#include "cono_map.h"
#include "widget_set.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "intl.h"

DrawingMethod drawing_method;
Parameter parameter;

/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
extern GtkTooltips *tooltips;

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget *SelectButtonFrame;
static GtkWidget *AxialTypeButton, *DrawingModeButton, *TestPlateButton,
                 *ImageRangeButton;

/*==================*
 * GLOBAL FUNCTIONS *
 *==================*/
void updateOpticFeatureFrame();
void updateOrientationFrame();
void updateOtherParametFrame();
void updateStereoFrame();

/*========================================* 
 * Internal Functions (Callback Functions)*
 *========================================*/
static void axial_type_change(GtkWidget *widget, gpointer data);
static void drawing_mode_change(GtkWidget *widget, gpointer data);
static void test_plate_change(GtkWidget *widget, gpointer data);
static void image_range_change(GtkWidget *widget, gpointer data);


/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *               createSelectButtonFrame                         *
 *===============================================================*/
void createSelectButtonFrame(GtkWidget *parent)
{
   GtkWidget *box = gtk_hbox_new(TRUE, 0);
   SelectButtonFrame = gtk_frame_new(NULL);
   gtk_box_pack_start(GTK_BOX(parent), SelectButtonFrame, FALSE, FALSE, 0);
   gtk_container_add(GTK_CONTAINER(SelectButtonFrame),  box);
   
   AxialTypeButton = pack_button(box, _("Uniaxial"));
   gtk_container_set_border_width (GTK_CONTAINER (AxialTypeButton), 0);   
   drawing_method.axial_type = Uniaxial;
   gtk_tooltips_set_tip (tooltips, AxialTypeButton, _("Axial Type"), NULL);

   DrawingModeButton = pack_button(box, _("Single Image"));
   gtk_container_set_border_width (GTK_CONTAINER (DrawingModeButton), 0);   
   drawing_method.drawing_kind = Single;			 
   gtk_tooltips_set_tip (tooltips, DrawingModeButton, _("Drawing Mode"), NULL);

   TestPlateButton = pack_button(box, _("Test Plate OUT"));
   gtk_container_set_border_width (GTK_CONTAINER (TestPlateButton), 0);   
   drawing_method.test_plate = Out;
   gtk_tooltips_set_tip (tooltips, TestPlateButton, _("Test Plate(530nm)"), NULL);

   ImageRangeButton = pack_button(box, _("Microscopic View"));
   gtk_container_set_border_width (GTK_CONTAINER (ImageRangeButton), 0);   
   drawing_method.drawing_range = WithinOpeningAngle;
   gtk_tooltips_set_tip (tooltips, ImageRangeButton, _("Image Range"), NULL);

   /**************** signal connections ******************************/
   gtk_signal_connect (GTK_OBJECT (AxialTypeButton), "clicked",
		       GTK_SIGNAL_FUNC (axial_type_change), NULL);
   gtk_signal_connect (GTK_OBJECT (DrawingModeButton), "clicked",
		       GTK_SIGNAL_FUNC (drawing_mode_change), NULL);
   gtk_signal_connect (GTK_OBJECT (TestPlateButton), "clicked",
		       GTK_SIGNAL_FUNC (test_plate_change), NULL);
   gtk_signal_connect (GTK_OBJECT (ImageRangeButton), "clicked",
		       GTK_SIGNAL_FUNC (image_range_change), NULL);
}

/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *               gtk_button_replace_label                        *
 *===============================================================*/
void gtk_button_replace_label(GtkWidget *button, gchar* str)
{
   GList *list, *t;
   GtkLabel* label;

   list = gtk_container_children(GTK_CONTAINER(button));
   t = g_list_nth(list, 0);
   label = GTK_LABEL(t->data);
   gtk_label_set(label, str);
}

/*========================================* 
 * (Callback Functions)                   *
 *                                        *
 *           axial_type_change            *
 *           drawing_mode_change          *
 *	     test_plate_change            *
 *           image_range_change           *  
 *                                        * 
 *========================================*/
   
static void
axial_type_change(GtkWidget *button, gpointer data)
{
   if(drawing_method.axial_type == Uniaxial)
   {
      drawing_method.axial_type = Biaxial;
      gtk_button_replace_label(button, _("Biaxial"));
   }
   else if(drawing_method.axial_type == Biaxial)
   {
      drawing_method.axial_type = Uniaxial;
      gtk_button_replace_label(button, _("Uniaxial"));
   }
   updateOrientationFrame();
   updateStereoFrame();
   updateOpticFeatureFrame();
}

static void
drawing_mode_change(GtkWidget *button, gpointer data)
{
   if(drawing_method.drawing_kind == Single)
   {
      drawing_method.drawing_kind = Circular;
      gtk_button_replace_label(button, _("Circular Image"));
   }
   else if(drawing_method.drawing_kind == Circular)
   {
      drawing_method.drawing_kind = Animation;
      gtk_button_replace_label(button, _("Animation"));
   }
   else if(drawing_method.drawing_kind == Animation)
   {
      drawing_method.drawing_kind = Single;
      gtk_button_replace_label(button, _("Single Image"));
   }
   updateOrientationFrame();
   updateStereoFrame();
   updateOtherParametFrame();
}

static void
test_plate_change(GtkWidget *button, gpointer data)
{
   if(drawing_method.test_plate == Out)
   {
      drawing_method.test_plate = In;
      gtk_button_replace_label(button, _("Test Plate IN"));
   }
   else if(drawing_method.test_plate == In)
   {
      drawing_method.test_plate = Out;
      gtk_button_replace_label(button, _("Test Plate OUT"));
   }
}

static void
image_range_change(GtkWidget *button, gpointer data)
{
   if(drawing_method.drawing_range == WithinOpeningAngle)
   {
      drawing_method.drawing_range = WholeAngle;
      gtk_button_replace_label(button, _("Full Range"));
   }
   else if(drawing_method.drawing_range == WholeAngle)
   {
      drawing_method.drawing_range = WithinOpeningAngle;
      gtk_button_replace_label(button, _("Microscopic View"));
   }
   updateStereoFrame();
}
   
